﻿--[[
	Persistent Data for RaidTracker, Saved Variables

	Aribrary definitions to keep things clear on identifiers:
		Group   - the set of 5 or less members in a party, raid or BG, or any set of members in any type of group
		Party   - group of 5 or less that is not in a raid and is using the Party UI
		Raid    - a set of one or more groups that has formed into a raid and is using the Raid UI
		BG      - a raid that is a pvp raid or party and is using the PvP UI (BattleGroup)

		Rarity - the quality or rarity of an item, quality and rarity are used interchangably
				 rarity is chosen for identifier because using word quality by self could be ambiguous
			[RarityID]  1:poor, 2:common, 3:uncommon, 4:rare, 5:epic, 6:legendary, 7:artifact
]]


RaidTracker = KarmaLibrary:GetInstance( "RaidTracker", {

} );

RaidTracker:RegisterDB( "KARaidTrackerDB", {
	Options = {
		AutoRaid = 2,                   -- [0:off|1:log|2:create]  logging raid if a raid
		AutoParty = 1,					-- [0:off|1:log|2:create]  logging raid if a party
		AutoSolo = 1,					-- [0:off|1:log|2:create]  logging raid if solo
		AutoBattlegroup = 0,			-- [0:off|1:log|2:create]  logging raid if a battelgroup
		AutoArena = 0,					-- [0:off|1:log|2:create]  logging raid if a battelgroup
		AutoEvent = 1,                  -- [0:off|1:onMouseover|2:onBossKill]  when to check for auto boss
		AutoZone = 1,                   -- [0|1]  automatically determine zone
		CostAsk = 0,                    -- [RarityID|0:off]  minimum item rarity to ask for dkp cost for items (ask user)
		CostGet = 0,                	-- [RarityID|0:off]  minimum item rarity to get for dkp cost for items (get from plugin)
		DebugMode = 0,                  -- [0|1]  Debug mode for development
		ExportFormat = 2,               -- [0:MLdkp1.1/EQdkpPlugin|1:EQdkpUnknown|2:MLdkp1.5]  Export format for dkp
		GroupItems = 3,                 -- [RarityID|0:off]  maximum item rarity to group items by count instead of log individualy
		LogAttendees = 1,               -- [0:off|1:onLoot|2:onBoss]  log attendees in database on specified event
		LogGuild = 1,                   -- [0|1]  Snapshots the guildroster on bosskill
		LogWipe	= 1,					-- [0:off|1:always|2..9:onAsk(10-80%)] ask if the group dies if it is a wipe, if all are dead it will not ask
		LevelMax = 80,                  -- If player lvl is maxlevel it will not be exported to mldkp
		LogItems = 2,                   -- [RarityID|0:off]
		SaveExtPlayer = 1,				-- on/off - save race, class and level
		SaveTooltips = 0,               -- on/off - save tooltips of items
		TimeFormat = 1,                 -- on/off - Use 24h time format
		Timezone = 0,					-- logging players timezone offset
		TimeSync = 1,
		-- Internal flags
		Color = "|c0076dfff",			-- optional text accent color
		ItemIcon = "INV_Misc_Gear_08",	-- default item icon, used if none exists
		ItemOptionsSelected = nil,      -- last selected item option
		CurrentRaid = nil,				-- raid id of raid currently beign logged
		BossName = "Trash mob",         -- just the name of the boss
		EventLast = 0,					-- timer to not initiate a new event
		EventCooldown = 120,			-- [seconds]  how long should trash mob's ignored after a boss kill
		WipeLast = 0,					-- timer to not bug if was a wipe again
		WipeCooldown = 150,             -- how long should death be ignored after a wipe count (seconds)
		VersionCopy = 0,                -- [0|1]  Is imported CT_RaidTracker, one time thing very first load only
		VersionFix = 17,                -- [0...]  Current database version level
	},
	SortOptions = {
		method = "name",
		way = "asc",
		itemsmethod = "looted",
		itemsway = "asc",
		itemsfilter = 1,
		playeritemsfilter = 1,
		playeritemsmethod = "name",
		playeritemsway = "asc",
		playerraidway = "desc",
		itemhistorymethod = "name",
		itemhistoryway = "asc",
	},
	ItemOptions = {
		-- BC
		[29434] = { id = 29434, name = "Badge of Justice",        Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 4, },
		[22450] = { id = 29434, name = "Void Crystal",            Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 4, },
		--   TK - Kael'thas
		[30312] = { id = 30311, name = "Warp Slicer",             Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 5, },
		[30312] = { id = 30312, name = "Infinity Blade",          Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 5, },
		[30313] = { id = 30313, name = "Staff of Disintegration", Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 5, },
		[30314] = { id = 30314, name = "Phaseshift Bulwark",      Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 5, },
		[30316] = { id = 30316, name = "Devastation",			  Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 5, },
		[30317] = { id = 30317, name = "Cosmic Infuser",          Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 5, },
		[30318] = { id = 30318, name = "Netherstrand Longbow",    Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 5, },
		[30319] = { id = 30319, name = "Nether Spike",            Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 5, },
		--   BT - Naj'entus
		[32408] = { id = 32408, name = "Naj'entus Spine",         Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 2, },
		-- WotLK
		[40752] = { id = 40752, name = "Emblem of Heroism",       Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 4, }, 
		[40753] = { id = 40753, name = "Emblem of Valor",         Track = 0, Group = 1, CostGet = 0, CostAsk = 0, quality = 4, }, 
	},
	Zones = RaidTracker_Zones,
	Bosses = RaidTracker_BossUnitTriggers,
	Online = { },
	Log = { },
} );


function RaidTracker:RunVersionFix( frame )
	local db = self._store.db;

	if db.Options.VersionCopy == 0 then
		if CT_RaidTracker_RaidLog then
			self:Print("Importing CT_RaidTracker database.")
			self:ImportToDB( "Options", _G, "CT_RaidTracker_", false, false, {
				"VersionFix",CurrentRaid="GetCurrentRaid","MinQuality","AutoRaidCreation","GroupItems",DkpValue="GetDkpValue","OldFormat",
				"AutoBoss","AutoBossBoss","AutoZone",CostAsk="AskCosts","Timezone","TimeSync",DebugMode="DebugFlag" } );
			self:ImportToDB( nil, _G, "CT_RaidTracker_", true, false, {"Options","ItemOptions","SortOptions"} );
			self:ImportToDB( nil, _G, "CT_RaidTracker_", false, false, {"RaidLog","CustomZoneTriggers","Online"} );
		end
		db.Options.VersionCopy = 1;
	end
		
	local opts = self._options;
	local log = db.RaidLog or db.Log;
	
	if opts.VersionFix == 0 then
		self:Debug("VersionFix", 1);
		for k, v in pairs(log) do
			if not log[k].PlayerInfos then log[k].PlayerInfos = { } end
			if v.Notes then
				for notesk, notesv in pairs(v["Notes"]) do
					if not log[k].PlayerInfos[notesk] then log[k].PlayerInfos[notesk] = { } end
					log[k].PlayerInfos[notesk]["note"] = notesv;
					self:Debug("VersionFix", 1, "note", k, notesk, notesv);
				end
				log[k]["Notes"] = nil;
			end
		end
		opts.VersionFix = 1;
	end
	if opts.VersionFix == 1 then
		self:Debug("VersionFix", 2);
		opts.VersionFix = 2; -- Do not remove tooltips any longer
	end
	if opts.VersionFix == 2 then
		self:Debug("VersionFix", 3);
		for k, v in pairs(log) do
			if log[k].BossKills and getn(log[k].BossKills) >= 1 and not log[k].BossKills[1] then
				local tempbosskills = {};
				for k2, v2 in pairs(log[k].BossKills) do tempbosskills[k2] = v2 end
				log[k].BossKills = {};
				for k2, v2 in pairs(tempbosskills) do
					tinsert(log[k].BossKills, { boss = k2, time = v2, attendees = {}, } );
				end
			end
		end
		opts.VersionFix = 3;
	end
	if opts.VersionFix == 3 then
		self:Debug("VersionFix", 4);
		for k, v in pairs(log) do
			if log[k].BossKills then
				for k2, v2 in pairs(log[k].BossKills) do
					if type(v2.time) == "table" then
						self:Debug("VersionFix", 4, "BossKills Fix", k, k2, v2.time.boss, v2.time.time);
						log[k].BossKills[k2].boss = v2.time.boss;
						log[k].BossKills[k2].attendees = v2.time.attendees;
						log[k].BossKills[k2].time = v2.time.time;
					end
				end
			end
		end
		opts.VersionFix = 4;
	end
	if opts.VersionFix == 4 then
		self:Debug("VersionFix", 5);
		opts.Wipe = 1; opts.WipePercent = 0.5; opts.NextBoss = 1;		
		opts.VersionFix = 5;
	end
	if opts.VersionFix == 5 then
		self:Debug("VersionFix", 6); opts.MLdkp = 1;		
		opts.VersionFix = 6;
	end;
	if opts.VersionFix == 6 then
		self:Debug("VersionFix", 7);
		if opts.MLdkp and opts.MLdkp == 1 then opts.ExportFormat = 2 else
			if opts.OldFormat and opts.OldFormat == 1 then opts.ExportFormat = 0 else opts.ExportFormat = 1 end
		end
		opts.MLdkp = nil; opts.OldFormat = nil;
		opts.VersionFix = 7;
	end;
-- Post CT RaidTracker updates, 1.5.5 from eris actually has soem changes marked as 8 as well, but they arnt big deal, dont conflict
	if opts.VersionFix == 7 then
		self:Debug("VersionFix", 8);
		self:ImportRenameDB( opts, { TimeFormat="24hFormat", CurrentRaid="GetCurrentRaid", DkpValue="GetDkpValue", } );

		opts.VersionFix = 8;
	end;
	if opts.VersionFix == 8 or opts.VersionFix == 9 then
		self:Debug("VersionFix", 9, 10);
		self:ImportRenameDB( db.SortOptions, { itemsfilter="itemfilter", playeritemsfilter="playeritemfilter", DebugMode="DebugFlag", } );
		if not opts.DebugMode then opts.DebugMode = 0; end

		opts.VersionFix = 10;
	end;
	if opts.VersionFix == 10 then
		self:Debug("VersionFix", 11);
		-- options
		self:ImportRenameDB( opts, { SavePlayerDetail="SaveExtendedPlayerInfo", CostAsk="AskCosts", CostGet="DkpValue", WipeAsk="Wipe",
			WipeCooldown="WipeCoolDown", AutoParty="LogGroup", AutoRaid="AutoRaidCreation", BossName="AutoBossBoss",
			BossCooldown="AutoBossChangeMinTime", LogGuild="GuildSnapshot", BossAsk="NextBoss", LogItems="MinQuality",
			AutoBattlegroup="LogBattlefield", } );
		if opts.AutoGroup == 1 then opts.AutoParty = 2 end; opts.AutoGroup = nil;
		if opts.AutoRaid == 1 then opts.AutoRaid = 2 end;
		if opts.AutoBattelgroup == 1 then opts.AutoArena = 1 end;
		-- item options
		local t = { Track="status", CostAsk="askcosts", CostGet="costgrabbing", Group="group", }
		for k, v in pairs(db.ItemOptions) do
			if type(v)=="table" then
				self:ImportRenameDB( v, t );
				table.remove(db.ItemOptions, k);
				db.ItemOptions[v.id] = v;
			end
		end
		-- zone triggers
		self:ImportToDB( nil, nil, nil, false, true, {ZoneTriggers="CustomZoneTriggers"} );
		-- raid log
		for k, v in pairs(log) do
			for k1, v1 in pairs(v.Loot) do
				if v1.player == "disenchanted" then v1.player = "disenchant" end
				v1.id = k1;
				if v1.thisitemid then v1.thisitemid = nil end
			end
		end

		opts.VersionFix = 11;
	end;
	if opts.VersionFix == 11 then
		self:Debug("VersionFix", 12);
		-- raid log
		local t = { bossnext="nextboss", }
		for k, v in pairs(log) do
			self:ImportRenameDB( v, t );
			for k1, v1 in pairs(v.Loot) do
				local c = tonumber(v1.item.count);
				if c==0 or c==1 or not c then v1.item.count = nil; end
			end
			if v.zone then
				if v.zone == "Tempest Keep: The Eye" then v.zone = "Tempest Keep" end
			end
		end

		opts.VersionFix = 12;
	end;
	if opts.VersionFix == 12 then
		self:Debug("VersionFix", 13);
		self:ImportRenameDB( _G, { RaidTrackerAceDB="KARaidTrackerAceDB" } );
		
		opts.VersionFix = 13;
	end;
	if opts.VersionFix == 13 then
		self:Debug("VersionFix", 14);
		db.CustomZoneTriggers = nil;
		self:ImportToDB( nil, nil, nil, false, true, { Zones="ZoneTriggers", Log="RaidLog" } ); log = db.Log;
		self:ImportRenameDB( opts, { "TimeOffsetStatus", LogWipe="WipeAsk", LevelMax="MaxLevel", AutoEvent="AutoBoss", SaveExtPlayer="SavePlayerDetail" } );
		if opts.LogWipe > 0 and opts.WipePercent then opts.LogWipe = min(floor(opts.WipePercent*10+2),9); end opts.WipePercent=nil;
		if opts.LevelMax == 70 then opts.LevelMax = 80 end
		if opts.BossAsk and opts.BossAsk == 1 then opts.AutoEvent = 3 end; opts.BossAsk = nil;

		local t = { Events="BossKills", Key="key" }
		for k, v in pairs(log) do
			self:ImportRenameDB( v, t );
			if v.Key and type(v.Key)~="number" then v.Key = self:GetTimestamp(v.Key) end
			if v.End and type(v.End)~="number" then v.End = self:GetTimestamp(v.End) end
			for k1, v1 in pairs(v) do
				if type(v1)=="table" then
					for k2, v2 in pairs(v1) do
						if type(v2)=="table" and v2.time and type(v2.time)~="number" then v2.time = self:GetTimestamp(v2.time) end
					end
				end
			end
			for k1, v1 in pairs(v.Loot) do
				local c = tonumber(v1.item.count);	if c==0 or c==1 or not c then v1.item.count = nil; end
				local c = v1.item.tooltip;			if c and (type(c)~="table" or #c==0) then v1.item.tooltip = nil; end
				local c = v1.attendees;				if c and (type(c)~="table" or #c==0) then v1.attendees = nil; end
			end
		end

		opts.VersionFix = 14;
	end;
	if opts.VersionFix == 14 then
		self:Debug("VersionFix", 15);
		for k, v in pairs(log) do
			for k1, v1 in pairs(v.Loot) do
				local c = tonumber(v1.item.count);	if c==0 or c==1 or not c then v1.item.count = nil; end
				local c = v1.item.tooltip;			if c and (type(c)~="table" or #c==0) then v1.item.tooltip = nil; end
				local c = v1.attendees;				if c and (type(c)~="table" or #c==0) then v1.attendees = nil; end
			end
		end
		opts.VersionFix = 15;
	end;
	if opts.VersionFix == 15 then
		self:Debug("VersionFix", 16);	
		self:ImportRenameDB( opts, { EventCooldown="BossCooldown", } );
		
		local t = { Zone="zone", Wipes="wipes", Players="PlayerInfos", }
		for k, v in pairs(log) do
			self:ImportRenameDB( v, t );
		end

		opts.VersionFix = 16;
	end;
	if opts.VersionFix == 16 then
		self:Debug("VersionFix", 17);	
		self:ImportRenameDB( db.SortOptions, { itemsfilter="itemfilter", itemsmethod="itemmethod", itemsway="itemway",
			playeritemsfilter="playeritemfilter", playeritemsmethod="playeritemmethod", playeritemsway="playeritemway", } );
			
		local t = { Zone="zone", Wipes="wipes", Players="PlayerInfos", }
		for k, v in pairs(log) do
			self:ImportRenameDB( v, t );
			if not v.Players then v.Players = { } end
		end

		opts.VersionFix = 17;
	end;

end
