﻿-- ********* KarmaStub lib versioning 1.0 start *********
if not KarmaStub or not KarmaStub.isLib("KarmaStub", 1.0) then
	KarmaStub = {
		list = KarmaStub and KarmaStub.list or { KarmaStub = 1.0, },
		isLib = function(self, lib, ver, rev)
			if rev then ver = (tonumber(ver) or 1) + ((tonumber(rev:match("%d+")) or 0) * 0.00000001) end
			if self.list[lib] and self.list[lib] >= ver then return _G[lib] else self.list[lib] = ver end
		end,
	}
end
-- ********* KarmaStub lib versioning 1.0 end ***********

if KarmaStub:isLib("KarmaUtility", 1.0, "$Revision: 99999 $") then return end

--[[
	General Utility Functions	
]]

KarmaUtility = {
	_lookup = {
		EscapeToChar = {
			["\194\161"] = "!",	["\194\170"] = "a", ["\194\186"] = "o", ["\194\191"] = "?", ["\195\128"] = "A", ["\195\129"] = "A",
			["\195\130"] = "A", ["\195\131"] = "A", ["\195\133"] = "A", ["\195\135"] = "C", ["\195\136"] = "E", ["\195\137"] = "E",
			["\195\138"] = "E", ["\195\139"] = "E", ["\195\140"] = "I", ["\195\141"] = "I", ["\195\142"] = "I", ["\195\143"] = "I",
			["\195\144"] = "D", ["\195\145"] = "N", ["\195\146"] = "O", ["\195\147"] = "O", ["\195\148"] = "O", ["\195\149"] = "O",
			["\195\152"] = "O", ["\195\153"] = "U", ["\195\154"] = "U", ["\195\155"] = "U", ["\195\157"] = "Y", ["\195\160"] = "a",
			["\195\161"] = "a", ["\195\162"] = "a", ["\195\163"] = "a", ["\195\165"] = "a", ["\195\167"] = "c", ["\195\168"] = "e",
			["\195\169"] = "e", ["\195\170"] = "e", ["\195\171"] = "e", ["\195\172"] = "i", ["\195\173"] = "i", ["\195\174"] = "i",
			["\195\175"] = "i", ["\195\176"] = "d", ["\195\177"] = "n", ["\195\178"] = "o", ["\195\179"] = "o", ["\195\180"] = "o",
			["\195\181"] = "o", ["\195\184"] = "o", ["\195\185"] = "u", ["\195\186"] = "u", ["\195\187"] = "u", ["\195\189"] = "y",
			["\195\191"] = "y", ["\195\132"] = "Ae", ["\195\134"] = "AE", ["\195\150"] = "Oe", ["\195\156"] = "Ue", ["\195\158"] = "TH",
			["\195\159"] = "ss", ["\195\164"] = "ae", ["\195\166"] = "ae", ["\195\182"] = "oe", ["\195\188"] = "ue", ["\195\190"] = "th",
		},
		ColorToRarity = {
			ff9d9d9d = 1, ffffffff = 2, ff1eff00 = 3, ff0070dd = 4, ffa335ee = 5, ffff8000 = 6, ffe6cc80 = 7,
		},
		ClassToId = {
			WARRIOR=1, ROGUE=2, HUNTER=3, PALADIN=4, SHAMAN=5, DRUID=6, WARLOCK=7, MAGE=8, PRIEST=9, DEATHKNIGHT = 10,
		},
		RaceToId = {
			Gnome=1, Human=2, Dwarf=3, NightElf=4, Troll=5, Scourge=6, Orc=7, Tauren=8,	Draenei=9, BloodElf=10,
		},
	},
	
	_options = { DebugMode = 0, },
};

function KarmaUtility:ToStr( d, sz, ... )
	local c,s,t,v = select("#",...),""
	for i=1,c do
		v = select(i,...);
		t = type(v);
		s = s .. (t=="string" and ((sz and #v>sz) and (strsub(v,1,sz).."...") or v)
			or ((t=="number" or t=="boolean") and tostring(v) or ("["..t.."]")))
		if d and c~=i then s = s .. d end
  	end	
	return s;
end

function KarmaUtility:Print( msg, r, g, b )
	DEFAULT_CHAT_FRAME:AddMessage(msg, r or 1, g or 1, b or 0);
end

function KarmaUtility:SysMsg( ... )
    self:Print(self:ToStr(" ",200,...), 1, 0.5, 0);
end

function KarmaUtility:Debug( ... )
    if self._options and self._options.DebugMode==1 then
		self:Print(self:ToStr("#",200,"  #",...).."#", 1, 0.5, 0)
		return true;
	end
	return false;
end


function KarmaUtility:ExecuteChatCommand( s )
	if not DEFAULT_CHAT_FRAME then return end
	DEFAULT_CHAT_FRAME.editBox:SetText( s );
	ChatEdit_SendText( DEFAULT_CHAT_FRAME.editBox );
end


function KarmaUtility:ArgsToTable( dst, ... )
	dst  = dst or { };
	for i = 1, select('#', ...), 2 do
		dst[select(i, ...)] = select(i+1, ...);
	end
	return dst;
end


function KarmaUtility:StripSpecialChars( s )
	for k,v in pairs(self._lookup.EscapeToChar) do
		s = string.gsub(s, k, v);	
	end
	return s;
end

function KarmaUtility:ConvertGlobalString( s )
	-- Stolen from nurfed (and fixed for german clients)
	s = string.gsub(s, "%%%d%$", "%%"); 
	s = string.gsub(s, "%%s", "(.+)");
	s = string.gsub(s, "%%d", "(%%d+)");
	return s;
end


function KarmaUtility:TableSet( t, k, v, isCopy )
	if rawget(t, k) ~= v then
		if t[k] == v then v = self:CopyDeep(v) end		-- exists
		t[k] = v;
	end
	return v;
end

function KarmaUtility:TableAddUnique( t, v )
	if type(t) ~= "table" then return end
	local found
	for k1, v1 in pairs(t) do
		if v == v1 then	found = true; break end
	end 
	if not found then table.insert(t, v) end
	return v;
end

function KarmaUtility:TableGetTables( t )
	local a,m = { }
	while type(t) == "table" and #a < 10 do
		tinsert(a, t)
		m = getmetatable(t);
		t = (m and m ~= t) and m.__index or nil;
	end	
	return a;
end

function KarmaUtility:TableGetKeys( t )
	local list, a = { }, self:TableGetTables(t);
	for i, v in ipairs( a ) do
		for k1, v1 in pairs( v ) do
			if type(v1)~="function" then
				self:TableAddUnique(list, k1)
			end
		end
	end
	return list, a;
end

function KarmaUtility:TableGetValues( t, sort )
	local list, keys, a = { }, self:TableGetKeys(t)
	for i, v in ipairs(keys) do
		tinsert( list, t[v] )
	end
	if sort then table.sort(list, sort) end
	return list;
end

function KarmaUtility:DebugTable( ... )
    if not self:Debug(...) then return end
	for i=1, select("#", ...), 1 do
		local idx, a = self:TableGetKeys( select(i,...) );
		table.sort( idx, self.sortSafe );
		for k1, v1 in pairs( idx ) do
			local s = "";
			for k2, v2 in pairs( a ) do
				s = self:ToStr(":",200, s, rawget(v2, v1));
			end	
			self:SysMsg("    ", v1, "=", s );
		end
	end
end


function KarmaUtility.sortSafe(a,b)
	if tonumber(a) and tonumber(b) then
		return tonumber(a) < tonumber(b)
	elseif tostring(a) and tostring(b) then 
		return tostring(a) < tostring(b)
	end
	return false;
end

function KarmaUtility:sortPairs(t,comparator)
	local sortedKeys = {};
	table.foreach(t, function(k,v) table.insert(sortedKeys,k) end);
	table.sort(sortedKeys,comparator);
	local i = 0;
	local function _f(_s,_v)
		i = i + 1;
		local k = sortedKeys[i];
		if k then
			return k,t[k];
		end
	end
	return _f,nil,nil;
end


-- convert string to timestamp or return timestamp as now
function KarmaUtility:GetTime( s, ts )
	local t = time();
	if s then 
		local _, _, mon, day, year, hr, min, sec = string.find(s, "(%d+)/(%d+)/(%d+) (%d+):(%d+):(%d+)");
		if not mon then return end
		t = date("*t", t);
		t.month = tonumber(mon);
		t.year = tonumber("20"..year);
		t.day = tonumber(day);
		t.hour = tonumber(hr);
		t.min = tonumber(min);
		t.sec = tonumber(sec);
		t = time(t);
	end
    return t + (ts or 0);
end

-- convert timestamp,[tzoffset], or now to formatted string
function KarmaUtility:GetDate( n, tz )
	local t = date("*t", (n or time()) + (tz and (tz * 3600) or 0));
	return string.format("%.2d/%.2d/%.2d %.2d:%.2d:%.2d", t.month,t.day,strsub(t.year, 3), t.hour,t.min,t.sec);
end

-- get game server time (current time of day for that servers timezone)	
function KarmaUtility:GetGameTime( )
	local hour, minute = GetGameTime();						-- get game server time (current time of day for that servers timezone)	
	return (hour * 3600) + (minute * 60);				
end

-- get this computers time (current time of day for this computers timezone)
function KarmaUtility:GetSysTime( )
	local t= date("*t");									-- get this computers time (current time of day for this computers timezone)
	return (t.hour * 3600) + (t.min * 60) + t.sec
end

function KarmaUtility:GetTimeDiff( game, sys )
	game = game or self:GetGameTime();
	sys = sys or self:GetSysTime();
	local t = 0;
	if game >= sys then
		sys = game - sys;
		t = sys + (sys >= (12*3600) and (-24*3600) or 0);
	elseif game < sys then
		sys = sys - game;
		if sys >= (12*3600) then
			t = (24*3600) - sys;
		else
			t = sys * -1;
		end
	end
	return t
end

-- generic scale any size hex string to decimal number
function KarmaUtility:HexToScale( s, o1, o2 )			-- these are both pretty heavy,
	if s and o1 then s = strsub(s,o1,o2) end			--  need to make sure not blow up and get some kind of color
	if not s or strlen(s) == 0 then return 1 end		--  this would be total fail boat if not, not happy place to see for debugger all tiem
	local n = tonumber(s, 16);
	local divs = ((2^((s:len()/2)*8)) - 1);
--	self:Debug("HexToScale", s, n, divs, n/divs)
	return (n) and (n / divs) or 1
end


function KarmaUtility:ColorToRGB( s )
	if s and strlen(s) > 6 then s = strsub(s,strlen(s)-5) end
    return self:HexToScale(s,1,2), self:HexToScale(s,3,4), self:HexToScale(s,5,6)
end


function KarmaUtility:CopyDeep(src)
    local lookup = { }
    local function _copy(src)
        if type(src) ~= "table" then
            return src
        elseif lookup[src] then
            return lookup[src]
        end
        local t = {}
        lookup[src] = t
        for i, v in pairs(src) do
            t[_copy(i)] = _copy(v)
        end
        return t
    end
    return _copy(src)
end

function KarmaUtility:MergeDeep(dst, src, useRef)
	useRef = useRef or true;
 	local function _canCopy(dst, src)
 		if not dst then	return true	end
 	 	if not src or
 	 		(type(dst) == "table" and type(src) ~= "table") or 
 	 		(type(dst) ~= "table" and type(src) == "table") then
 			return false
 		end
	 	return true
	end
	local function _copy(dst, src)
		for i, v in pairs(src) do
			if dst[i] ~= v and _canCopy(dst[i], v) then
--				self:Debug(i, dst[i], v);
				if type(v) ~= "table" or (useRef and not dst[i]) then 
					dst[i] = v;
				else
					if not dst[i] then dst[i] = { } end
					_copy(dst[i], v)
				end
			end 	
		end
		return dst
	end
	if _canCopy(dst, src) then 
		if type(src) ~= "table" then return src end
		if not dst then dst = { } end 	
	 	_copy(dst, src)
	end
	return dst;
end

function KarmaUtility:GetScanTooltip( )
	if Karma_ScanTooltip then return Karma_ScanTooltip end
	f = CreateFrame( "GameTooltip", "Karma_ScanTooltip" ); -- Tooltip name cannot be nil
	f:SetOwner( WorldFrame, "ANCHOR_NONE" );
	f.left, f.right = { }, { }
	for i = 1, 30 do
		f.left[i] = f:CreateFontString( "$parentTextLeft..i", nil, "GameTooltipText" );
		f.right[i] = f:CreateFontString( "$parentTextRight..i", nil, "GameTooltipText" );
		f:AddFontStrings( f.left[i], f.right[1] );
	end
	return f;
end

function KarmaUtility:UnitZone( unit )
	local f = self:GetScanTooltip();
	f:ClearLines();
	f:SetUnit(unit);
	local s = f.left[f:NumLines()]:GetText();
	if s == PVP or s:find("^Level") then return end 
	return s;
end

function KarmaUtility:GetItemTooltip( link )
	local f, t = self:GetScanTooltip(), { };
	f:ClearLines();
	f:SetHyperlink( strfind(link,"^item:%d") and link or ("item:"..link) );
	for i = 1, f:NumLines() do
		tinsert(t, { left = f.left[i]:GetText(), right = f.right[i]:GetText() } );
	end
	return t;
end
